#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <linux/major.h>
#include "global.h"
#include "devnames.h"

DiskList *dlist = NULL;

const char *stripdev(const char *s) {
  if (strncmp(s,"/dev",4) == 0) s+= 4;
  if (s[0] == '/') s++;
  return s;
}

DiskList *find_dev(unsigned major, unsigned minor) {
  DiskList *dl;
  for (dl = dlist; dl; dl = dl->next) {
    if (dl->major == major && dl->minor == minor) return dl;
  }
  return NULL;
}

DiskList *find_dev_by_name(const char *name) {
  DiskList *dl;
  const char *sname = stripdev(name);
  for (dl = dlist; dl; dl = dl->next) {
    if (strcmp(sname, dl->name)==0 || strcmp(sname, dl->devfsname)==0) return dl;
  }
  return NULL;
}

DiskList *next_hd_in_list(DiskList *prev) {
  DiskList *dl = (prev ? prev->next : dlist);
  while (dl && is_partition_name(dl->name)) dl = dl->next;
  return dl;
}

DiskList *first_hd_in_list() {
  return next_hd_in_list(NULL);
}

int nb_hd_in_list() {
  int i=0;
  DiskList *dl;
  for (dl = first_hd_in_list(); dl; dl = next_hd_in_list(dl)) ++i;
  return i;
}

int nb_dev_in_list() {
  int i=0;
  DiskList *dl;
  for (dl = dlist; dl; dl = dl->next) ++i;
  return i;
}

int is_partition_name(const char *name) {
  int l = strlen(name);
  if (l && name[l-1] >= '0' && name[l-1] <= '9') return 1;
  return 0;
}


/* stolen from hdparm sources ..*/
void add_device(const char *devname) {
  struct stat stat_buf;
  DiskList *dl; 

  ALLOC_OBJ(dl);
  dl->name = strdup(stripdev(devname));
  dl->devfsname = strdup(stripdev(devname));
  dl->is_scsi = 0;
  dl->next = dlist; dlist = dl;
  if (lstat(devname,&stat_buf)) { perror(devname); exit(errno); }
  if (S_ISLNK(stat_buf.st_mode)) {
    char lname[512];
    int n = readlink(devname, lname, 511); lname[n] = 0;
    free(dl->devfsname); dl->devfsname = strdup(stripdev(lname));
    if (stat(devname,&stat_buf)) { perror(devname); exit(errno); }
  }
  if (!S_ISBLK(stat_buf.st_mode)) {
    fprintf(stderr, "%s is not a block device..\n", devname);
    exit(1);
  }
  dl->major = major(stat_buf.st_rdev);
  dl->minor = minor(stat_buf.st_rdev);
  BLAHBLAH(1,printf("stat(%s) : major=%d, minor=%d, devfsname = %s\n", devname, dl->major, dl->minor, dl->devfsname));
  /* find if it is an scsi device */
  switch(dl->major) {
#ifdef SCSI_DISK0_MAJOR
  case (SCSI_DISK0_MAJOR):
  case (SCSI_DISK1_MAJOR):
  case (SCSI_DISK2_MAJOR):
  case (SCSI_DISK3_MAJOR):
  case (SCSI_DISK4_MAJOR):
  case (SCSI_DISK5_MAJOR):
  case (SCSI_DISK6_MAJOR):
  case (SCSI_DISK7_MAJOR):
#else
  case (SCSI_DISK_MAJOR):
#endif
#ifdef MD_MAJOR
  case (MD_MAJOR):
#endif
  case (VXVM_MAJOR):
#ifdef LVM_BLK_MAJOR
  case (LVM_BLK_MAJOR):
#endif
#ifdef CCISS_MAJOR
  case (CCISS_MAJOR):
#endif
    dl->is_scsi = 1;
    break;
  case IDE0_MAJOR:
  case IDE1_MAJOR:
#ifdef IDE2_MAJOR
  case IDE2_MAJOR:
#endif
#ifdef IDE3_MAJOR
  case IDE3_MAJOR:
#endif
#ifdef IDE4_MAJOR
  case IDE4_MAJOR:
#endif
#ifdef IDE5_MAJOR
  case IDE5_MAJOR:
#endif
#ifdef IDE6_MAJOR
  case IDE6_MAJOR:
#endif
#ifdef IDE7_MAJOR
  case IDE7_MAJOR:
#endif
#ifdef IDE8_MAJOR
  case IDE8_MAJOR:
#endif
#ifdef IDE9_MAJOR
  case IDE9_MAJOR:
#endif
    break;	/* do nothing */
  default:
    fprintf(stderr,"device %s is not supported\n",devname);
    exit(EINVAL);
    break;
  }
  dl->enable_hddtemp = 1;
  dl->enable_power_status = !dl->is_scsi;
}

strlist *swaplst = NULL;
void add_swap(const char *swapdev) {
  swaplst = strlist_ins(swaplst, swapdev);
}
strlist *swap_list() {
  return swaplst;
}
